<?php
/**
 * Plugin Name: iwocaPay Payment Gateway
 * Description: B2B Payments
 * Plugin URI: https://www.iwoca.co.uk/iwocapay-sellers/
 * Version: 0.15
 * Author: iwoca
 * Author URI: https://www.iwoca.co.uk/
 * 
 * @package iwocaPay WooCommerce Plugin
 */
/*
 * The class itself, please note that it is inside plugins_loaded action hook
 */
add_action( 'plugins_loaded', 'iwoca_init_gateway_class' );
function iwoca_init_gateway_class() {

    class WC_IWOCA_Gateway extends WC_Payment_Gateway {

        public function __construct() {
            // payment gateway plugin ID
            $this->id = 'iwocapay'; 
             // URL of the icon that will be displayed on the checkout page
             // TODO we should replace this with a static image, shipped with the plugin.
            $plugin_dir = plugin_dir_url(__FILE__);
            $this->icon = apply_filters( 'woocommerce_gateway_icon', $plugin_dir . '\assets\iwp-logo-small.svg' );
            // We don't use fields, just a button to continue to iwocaPay.
            $this->has_fields = false; 
            // The name and of the plugin in the admin
            $this->method_title = 'iwocaPay';
            $this->method_description = 'Let your business customers Pay Now or Pay Later';

            // Initialise the form fields in the Admin view.
            $this->init_form_fields();
            // Load the settings.
            $this->init_settings();

            // Assign some common variables that will be helpful for us.
            $this->title = $this->get_option( 'title' );
            $this->description = $this->get_option( 'description' );
            $this->enabled = $this->get_option( 'enabled' );
            $this->is_test_mode = 'yes' === $this->get_option( 'is_test_mode' );
            $this->seller_id = $this->is_test_mode ? $this->get_option('stage_seller_id') : $this->get_option('production_seller_id');
            $this->api_key = $this->is_test_mode ? $this->get_option('stage_api_key') : $this->get_option('production_api_key');
            $this->payment_modes = $this->get_option( 'payment_modes' );
            $this->base_url = $this->is_test_mode ? 'https://stage.iwoca-dev.co.uk/api/lending/edge' : 'https://www.iwoca.co.uk/api/lending/edge';
            
            // Register the callback url
            add_action( 'woocommerce_api_iwocapay', array( $this, 'callback' ) );

            // This action hook saves the settings
            add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
        }

        public function init_form_fields(){
            // This configures our form fields in the Admin.
            $this->form_fields = array(
                'enabled' => array(
                    'title'       => 'Enable/Disable',
                    'label'       => 'Enable iwocaPay',
                    'type'        => 'checkbox',
                    'description' => '',
                    'default'     => 'no'
                ),
                'title' => array(
                    'title'       => 'Title',
                    'type'        => 'text',
                    'description' => 'This controls the title the user sees during checkout.',
                    'default'     => 'Pay in 3',
                ),
                'description' => array(
                    'title'       => 'Description',
                    'type'        => 'textarea',
                    'description' => 'This controls the description the user sees during checkout.',
                    'default'     => 'Credit option for LTD companies, powered by iwocaPay',
                ),
                'payment_modes' => array(
                    'title'       => 'Payment modes',
                    'type'        => 'select',
                    'options'     => array (
                        'all_modes' => 'Pay later & Pay now',
                        'pay_later' => 'Pay later only',
                    ),
                    'description' => 'This controls the available checkout methods for customers.',
                    'default' => 'all_modes',
                ),
                'production_api_key' => array(
                    'title'       => 'API Key',
                    'type'        => 'password'
                ),
                'production_seller_id' => array(
                    'title'       => 'Seller ID',
                    'type'        => 'text'
                ),
                'is_test_mode' => array(
                    'title'       => 'Mode',
                    'label'       => 'Sandbox Mode',
                    'type'        => 'checkbox',
                    'description' => 'Uses Sandbox URL/Credentials if in test mode, otherwise Production.',
                    'default'     => 'no'
                ),
                'stage_api_key' => array(
                    'title'       => 'Sandbox API Key',
                    'type'        => 'password'
                ),
                'stage_seller_id' => array(
                    'title'       => 'Sandbox Seller ID',
                    'type'        => 'text'
                )
            );
        }

        private function create_iwocapay_order( $woocommerce_order ) {
            $payment_modes = ['PAY_LATER', 'PAY_NOW'];
            if ($this->payment_modes == 'pay_later') {
                $payment_modes = ['PAY_LATER'];
            }

            // Call iwocaPay API to create an order.
            $api_body = array(
                'amount' => floatval($woocommerce_order->get_total()),
                'reference' => '#' . $woocommerce_order->get_id(),
                'allowed_payment_terms' => $payment_modes,
                'source' => 'iwocaPay_woocommerce_extension',
                'redirect_url' => site_url() . '?wc-api=iwocapay',
            );
            $url = $this->base_url . '/ecommerce/seller/' . $this->seller_id . '/order/';
            $response = wp_remote_post( 
                $url, 
                array(
                'method'      => 'POST',
                'headers' => array(
                    'Accept' => 'application/json',
                    'Content-Type' => 'application/json; charset=utf-8',
                    'Authorization' => 'Bearer ' . $this->api_key
                ),
                'timeout'     => 45,
                'redirection' => 5,
                'httpversion' => '1.0',
                'blocking'    => true,
                'body'        => json_encode(
                    array(
                        'data' => $api_body
                    )
                )
            ));
            $is_error = is_wp_error( $response );
            $status_code = wp_remote_retrieve_response_code( $response );

            if ( $is_error || $status_code != 201 ) {
                return null;
            }

            $response_body = json_decode(wp_remote_retrieve_body( $response ));
            return $response_body;
        }

        private function fetch_iwocapay_order( $order_id ) {
            $url = $this->base_url . '/ecommerce/order/' . $order_id . '/';
            $response = wp_remote_get( 
                $url, 
                array(
                'method'      => 'GET',
                'headers' => array(
                    'Accept' => 'application/json',
                    'Content-Type' => 'application/json; charset=utf-8',
                    'Authorization' => 'Bearer ' . $this->api_key
                ),
                'timeout'     => 45,
                'redirection' => 5,
                'httpversion' => '1.0',
                'blocking'    => true,
            ));
            $is_error = is_wp_error( $response );
            $status_code = wp_remote_retrieve_response_code( $response );
            if ( $is_error || $status_code != 200 ) {
                return null;
            }

            $response_body = json_decode(wp_remote_retrieve_body( $response ));
            return $response_body;
        }

        public function process_payment( $order_id ) {
            /* The function called when a payment is initiated using our gateway.
            We simply make a request to the iwocaPay API, grab the url, and
            redirect the customer to that page.
            */

            $woocommerce_order = wc_get_order( $order_id );
            // Make the iwocaPay API request to create an order associated with this order
            $iwocapay_order = $this->create_iwocapay_order($woocommerce_order);
            
            if ($iwocapay_order == null) {
                // Something went wrong with the API request to iwocaPay, let the user know and exit.
                wc_add_notice( 'Something went wrong. Please try again.', 'error' );
                return array(
                    'result' => 'failure'
                );
            }

            // Redirect the user onto iwocaPay.
            $redirect_url = $iwocapay_order->data->order_url;
            return array(
                'result' => 'success',
                'redirect' => $redirect_url
            );
        }

        private function unsuccessful ($msg = null) {
            $msgToDisplay = $msg != null ? $msg : 'Sorry something went wrong. Please try again.';
            wc_add_notice( $msgToDisplay , 'error' );
            wp_redirect(wc_get_checkout_url());
            exit;
        }

        public function callback () {
            /* 
            This is where customers will be redirected to when iwocaPay is finished. 
            Orders are not necessarily successful at this point, so we will:
              * Fetch the order (based on the id in a query parameter)
              * Direct to the success page in the success case
              * Direct to the payment page in all other cases
            */
            $order_id = isset($_REQUEST['orderId']) ? sanitize_text_field($_REQUEST['orderId']) : null;
            if ( $order_id  == null ) {
                $this->unsuccessful();
            }

            $iwocapay_order = $this->fetch_iwocapay_order( $order_id );

            if ( $iwocapay_order  == null || !wp_is_uuid($order_id)) {
                $this->unsuccessful();
            }

            $iwocapay_order_reference = $iwocapay_order->data->reference;
            $iwocapay_order_status = $iwocapay_order->data->status;
            $woocommerce_order_id = intval(ltrim($iwocapay_order_reference, '#'));
            $woocommerce_order = wc_get_order($woocommerce_order_id);

            // Handle depending on status.
            if ( $iwocapay_order_status == 'SUCCESSFUL' ) {
                $woocommerce_order->payment_complete();
                wp_redirect($this->get_return_url($woocommerce_order));
                exit;
            } else {
                $this->unsuccessful('Sorry something went wrong, we were unable to process your payment.');
            }
        }
    }
}

/*
 * This action hook registers our PHP class as a WooCommerce payment gateway
 */
add_filter( 'woocommerce_payment_gateways', 'iwoca_add_gateway_class' );
function iwoca_add_gateway_class( $gateways ) {
    $gateways[] = 'WC_IWOCA_Gateway';
    return $gateways;
}

/*
 * This hook disables the iwocaPay gateway if cart is outside of min/max amount, for pay later only.
 */
add_filter( 'woocommerce_available_payment_gateways', 'filter_payment_gateway' );
function filter_payment_gateway( $available_gateways ) {
    $PAY_LATER_MINIMUM = 150.0;
    $PAY_LATER_MAXIMUM = 15000.0;

    $payment_modes = $available_gateways['iwocapay']->payment_modes;
    $cart_total = (float)WC()->cart->cart_contents_total;

    if ( $payment_modes == 'pay_later' && ($cart_total < $PAY_LATER_MINIMUM || $cart_total > $PAY_LATER_MAXIMUM )) {
        unset( $available_gateways['iwocapay'] );
    }
    return $available_gateways;
 }